@@ -32,8 +32,8 @@
 
 (eval-when-compile
   (require 'cl)
-  (require 'skk-macs)
-  (require 'static))
+  (require 'skk-vars)
+  (require 'skk-macs))
 
 (eval-when-compile
   (defvar skk-dcomp-start-point)
@@ -62,7 +62,7 @@
 $BByE@$HH>ByE@$rF~NO$9$k$?$a$N%k!<%k!#(B")
 
 (defconst skk-kanagaki-print-help-function
-  (cond ((and (string-match "^GNU" (emacs-version))
+  (cond ((and skk-running-gnu-emacs
 	      (>= emacs-major-version 23))
 	 #'help-print-return-message)
 	(t
