@@ -207,9 +207,6 @@ US 101 $B%-!<%\!<%I$G(B NICOLA $BF~NO$9$k$?$a$N4pK\%k!<%k!#(B")
 	    #'(lambda ()
 		(define-key skk-j-mode-map "'" 'skk-kanagaki-bs))))
 
-(require 'product)
-(product-provide
-    (provide 'skk-nicola-us)
-  (require 'skk-version))
+(provide 'skk-nicola-us)
 
 ;;; skk-nicola-us.el ends here
