@@ -44,16 +44,13 @@
   (require 'cl)
   (require 'skk-kanagaki-util)
   (require 'skk-macs)
-  (require 'skk-vars)
-  (require 'static))
+  (require 'skk-vars))
 
 (eval-and-compile
   (require 'skk-kanagaki))
 
-(static-when (and (string-match "^GNU" (emacs-version))
-		  (>= emacs-major-version 21))
-  (eval-and-compile
-    (require 'skk-e21)))
+(when (eval-when-compile skk-running-gnu-emacs)
+  (require 'skk-emacs))
 
 (eval-and-compile
   (autoload 'skk-dcomp-marked-p "skk-dcomp")
@@ -461,13 +458,13 @@
 	    ?\ )
 	(call-interactively 'self-insert-command t))
     ;; else
-    (let ((last (static-cond
-		 ((featurep 'xemacs)
+    (let ((last (cond
+		 ((eval-when-compile (featurep 'xemacs))
 		  (event-key last-command-event))
 		 (t
 		  last-command-event)))
-	  (next (static-cond
-		 ((featurep 'xemacs)
+	  (next (cond
+		 ((eval-when-compile (featurep 'xemacs))
 		  (event-key (next-command-event)))
 		 (t
 		  (next-command-event))))
@@ -566,7 +563,7 @@
 
 (defun skk-nicola-format-time (time)
   "`current-time' $B$NJV$97k2L$rJQ49$7$FI>2A$G$-$k$h$&$K$9$k!#(B"
-  (let ((time1 (* (float 65536) ;; 2x10^6
+  (let ((time1 (* (float 65536) ;; 2^16
 		  (car time)))
 	(time2 (cadr time))
 	(time3 (/ (caddr time)
@@ -575,8 +572,8 @@
 
 (defun skk-nicola-event-to-key (event)
   "EVENT $B$rH/@8$9$k%-!<$r<hF@$9$k!#(B"
-  (static-cond
-   ((featurep 'xemacs)
+  (cond
+   ((eval-when-compile (featurep 'xemacs))
     (let ((char (event-to-character event)))
       (if (characterp char)
 	  char
