@@ -5,10 +5,10 @@
 
 ;; Author: NAKAJIMA Mikio <minakaji@osaka.email.ne.jp>
 ;; Maintainer: SKK Development Team <skk@ring.gr.jp>
-;; Version: $Id: skk-annotation.el,v 1.159 2010/08/27 10:42:17 skk-cvs Exp $
+;; Version: $Id: skk-annotation.el,v 1.167 2010/09/26 04:23:13 skk-cvs Exp $
 ;; Keywords: japanese, mule, input method
 ;; Created: Oct. 27, 2000.
-;; Last Modified: $Date: 2010/08/27 10:42:17 $
+;; Last Modified: $Date: 2010/09/26 04:23:13 $
 
 ;; This file is part of Daredevil SKK.
 
@@ -120,9 +120,7 @@
 ;; $B$N9`L\$+$iC5$7!$8+$D$+$C$?>l9g$O!"FbMF$NH4?h$r%"%N%F!<%7%g%s$H$7$FI=<((B
 ;; $B$7$^$9!#$3$N5!G=$O(B Emacs 22 $B$G%F%9%H$5$l$F$$$^$9!#(BXEmacs 21.5 $B$G$O0J2<(B
 ;; $B$N(B 1 $B$H(B 2 $B$rF3F~$9$kI,MW$,$"$j$^$9!#(BXEmacs 21.4 $B$G$O99$K(B 3 $B$bI,MW$G$9!#(B
-;; Emacs 21.4 $B$G$b(B 1, 2, 3 $B$,I,MW$H$J$j$^$9!#(BEmacs 20.7 $B$G$NF0:n$O%5%]!<%H(B
-;; $B$7$^$;$s$,!"(BMULE 4.1 $B%Q%C%A$rEv$F$F$$$l$P(B Emacs 21 $B$HF1MM$KF0:n$9$k2DG=(B
-;; $B@-$,$"$j$^$9!#(B
+;; Emacs 21.4 $B$G$b(B 1, 2, 3 $B$,I,MW$H$J$j$^$9!#(B
 ;;
 ;; 1. html2text.el
 ;;
@@ -223,17 +221,11 @@
   (autoload 'url-retrieve "url"))
 
 (eval-when-compile
-  (require 'static)
-
   (defvar mule-version)
   (defvar html2text-remove-tag-list)
-  (defvar html2text-format-tag-list)
+  (defvar html2text-format-tag-list))
 
-  (when (and (string-match "^GNU" (emacs-version))
-	     (= emacs-major-version 20))
-    (defalias 'skk-tooltip-show-at-point 'ignore)))
-
-(static-when (featurep 'xemacs)
+(when (eval-when-compile (featurep 'xemacs))
   (require 'skk-xemacs))
 
 (unless skk-annotation-mode-map
@@ -252,22 +244,22 @@
 	(cons (cons 'skk-annotation-mode skk-annotation-mode-map)
 	      minor-mode-map-alist)))
 
-;; inline functions.
-(defsubst skk-annotation-erase-buffer ()
+;; functions.
+(defun skk-annotation-erase-buffer ()
   (let ((inhibit-read-only t)
 	buffer-read-only)
-    (static-when
-	(fboundp 'set-text-properties)
-      (set-text-properties (point-min) (point-max) nil))
+    (set-text-properties (point-min) (point-max) nil)
     (erase-buffer)))
 
-(defsubst skk-annotation-insert (annotation)
+(defun skk-annotation-insert (annotation)
   (with-current-buffer (get-buffer-create skk-annotation-buffer)
     (skk-annotation-erase-buffer)
+    (setq buffer-read-only nil)
     (insert annotation)
-    (goto-char (point-min))))
+    (goto-char (point-min))
+    (set-buffer-modified-p nil)
+    (setq buffer-read-only t)))
 
-;; functions.
 ;;;###autoload
 (defun skk-annotation-get (annotation)
   (if (string= annotation "")
@@ -347,9 +339,7 @@
 		      (setq event (next-command-event)
 			    key (skk-event-key event)
 			    command (key-binding
-				     (static-if (featurep 'xemacs)
-					 event
-				       key)))
+				     (if (featurep 'xemacs) event key)))
 		      ;; Return value of the following expression is important.
 		      (or (memq command list)
 			  (eq command 'digit-argument)
@@ -360,11 +350,10 @@
 				 (key-description
 				  skk-annotation-wikipedia-key))))
 		  (quit
-		   (static-when
-		       (and (featurep 'xemacs)
-			    (= emacs-major-version 21)
-			    (<= emacs-minor-version 4))
-		     ;; workaround
+		   (when (eval-when-compile (and (featurep 'xemacs)
+						 (= emacs-major-version 21)
+						 (= emacs-minor-version 4)))
+		     ;; workaround for XEmacs 21.4
 		     (keyboard-quit)))))
       (cond ((eq command copy-command)
 	     (setq list (delq copy-command list))
@@ -380,25 +369,24 @@
 	     (setq list (delq browse-command list))
 ;	     (setq urls (delq nil (mapcar #'skk-annotation-find-url notes)))
 	     (when word
-	       (cond ((setq cache
-			    (skk-annotation-wikipedia-cache word sources))
-		      (setq urls
-			    (cons (apply
-				   #'skk-annotation-generate-url
-				   "http://%s.org/wiki/%s"
-				   ;; split-string $B$NHs8_49@-$KG[N8(B
-				   (static-if (and (string-match
-						    "^GNU"
-						    (emacs-version))
-						   (<= emacs-major-version 21))
-				       (cdr (split-string (cdr cache) " "))
-				     (cdr (split-string (cdr cache) " " t))))
-				  urls)))
-		     (skk-annotation-show-wikipedia-url
-		      (add-to-list 'urls
-				   (skk-annotation-generate-url
-				    "http://ja.wikipedia.org/wiki/%s"
-				    word)))))
+	       (cond
+		((setq cache (skk-annotation-wikipedia-cache word sources))
+		 (setq urls
+		       (cons
+			(apply #'skk-annotation-generate-url
+			       "http://%s.org/wiki/%s"
+			       ;; split-string $B$NHs8_49@-$KG[N8(B
+			       (if (eval-when-compile
+				     (and skk-running-gnu-emacs
+					  (<= emacs-major-version 21)))
+				   (cdr (split-string (cdr cache) " "))
+				 (cdr (split-string (cdr cache) " " t))))
+			urls)))
+		(skk-annotation-show-wikipedia-url
+		 (add-to-list 'urls
+			      (skk-annotation-generate-url
+			       "http://ja.wikipedia.org/wiki/%s"
+			       word)))))
 	     (unless (equal annotation "")
 	       (cond
 		(urls
@@ -414,11 +402,12 @@
 		     char  nil)
 	       (skk-annotation-show-2 annotation)))
 	    ((eq command 'digit-argument)
-	     (setq char  (static-if (featurep 'xemacs)
-			     key
-			   (if (integerp event)
-			       event
-			     (get event 'ascii-character)))
+	     (setq char  (cond ((featurep 'xemacs)
+				key)
+			       ((integerp event)
+				event)
+			       (t
+				(get event 'ascii-character)))
 		   digit (- (logand char ?\177) ?0)
 		   event nil))
 	    ((or (equal (key-description key)
@@ -479,7 +468,8 @@
 	(when (equal key-wiki "TAB")
 	  (setq key-wiki "C-i"))
 	(setq skk-annotation-message
-	      (format "{$B%"%N%F!<%7%g%sA`:n(B}[%s]$B%3%T!<$9$k(B [%s]URL$B$r8+$D$1$k(B [%s]$B%G%U%)%k%H(BWiki$B$+$iH4?h(B"
+	      (format "\
+\{$B%"%N%F!<%7%g%s(B}[%s]$B%3%T!<(B [%s]URL$B%V%i%&%:(B [%s]$B%G%U%)%k%H(BWiki$B;2>H(B"
 		      key-copy key-browse key-wiki))))
     (condition-case nil
 	(cond ((eq situation 'annotation)
@@ -537,19 +527,26 @@
 	(let ((minibuf-p (skk-in-minibuffer-p))
 	      event window)
 	  (skk-annotation-insert annotation)
-	  (cond
-	   (minibuf-p
-	    (if (setq window (get-buffer-window (skk-minibuffer-origin)))
-		(select-window window)
-	      (other-window 1))
-	    (unless (eq (next-window) (selected-window))
-	      (delete-other-windows)))
-	   (t
-	    (split-window-vertically)))
+	  (cond (minibuf-p
+		 (if (setq window (get-buffer-window (skk-minibuffer-origin)))
+		     (select-window window)
+		   (other-window 1))
+		 (unless (eq (next-window) (selected-window))
+		   (delete-other-windows)))
+		(t
+		 (split-window-vertically)))
+	  ;;
 	  (display-buffer skk-annotation-buffer)
 	  (when minibuf-p
 	    (select-window (minibuffer-window)))
 	  ;;
+	  (unless (eval-when-compile (and (featurep 'xemacs)
+					  (= emacs-major-version 21)
+					  (<= emacs-minor-version 4)))
+	    (save-selected-window
+	      (pop-to-buffer skk-annotation-buffer)
+	      (fit-window-to-buffer)))
+	  ;;
 	  (skk-annotation-message 'annotation)
 	  ;;
 	  (setq event (next-command-event))
@@ -624,7 +621,7 @@ no-previous-annotation $B$r;XDj$9$k$H(B \(C-u M-x skk-annotation-add $B$G;XDj
     (skk-annotation-setup)
     (let* ((plist (append
 		   '(intangible t read-only t)
-		   (static-if (featurep 'xemacs)
+		   (if (featurep 'xemacs)
 		       '(start-closed t end-open t)
 		     '(front-sticky t rear-nonsticky t))))
 	   (wholestring (nth 2 skk-annotation-target-data))
@@ -650,8 +647,7 @@ no-previous-annotation $B$r;XDj$9$k$H(B \(C-u M-x skk-annotation-add $B$G;XDj
 ;; Add a note to word `%s' (this line will not be added to the note.)
 "
 	       realword))
-      (static-if (fboundp 'set-text-properties)
-	  (add-text-properties (point-min) (1- (point)) plist))
+      (add-text-properties (point-min) (1- (point)) plist)
       (when (and (not no-previous-annotation)
 		 annotation)
 	(insert annotation))
@@ -1468,14 +1464,15 @@ Wikipedia\\(</a>\\)? has an article on:$" nil t)
 		(throw 'found (cons string cache-buffer))))))))))
 
 ;;;###autoload
-(defun skk-annotation-wikipedia-region-or-at-point (&optional prefix-arg start end)
+(defun skk-annotation-wikipedia-region-or-at-point (&optional prefix-arg
+							      start end)
   "$BA*BrNN0h$^$?$O%]%$%s%H0LCV$NC18l$r(B Wikipedia/Wikitionary $B$GD4$Y$k!#(B
-$BNN0h$,A*Br$5$l$F$$$J$1$l$PC18l$N;O$a$H=*$o$j$r?dB,$7$FD4$Y$k!#(B"
+$BNN0h$,A*Br$5$l$F$$$J$1$l$PC18l$N;O$a$H=*$o$j$r?dB,$7$FD4$Y$k!#(B
+$BD4$Y$?7k2L$r(B skk-annotation-show-as-message $B$,(B Non-nil $B$G$"$l$P%(%3!<%(%j%"$K!"(B
+nil $B$G$"$l$PJL(B window $B$KI=<($9$k!#(B"
   (interactive (cons (prefix-numeric-value current-prefix-arg)
 		     (cond
-		      ((static-if (featurep 'xemacs)
-			   (region-active-p)
-			 (and transient-mark-mode mark-active))
+		      ((skk-region-active-p)
 		       (list (region-beginning) (region-end)))
 		      ((eq skk-henkan-mode 'on)
 		       (list (marker-position skk-henkan-start-point)
@@ -1526,37 +1523,36 @@ Wikipedia\\(</a>\\)? has an article on:$" nil t)
 
 (defun skk-annotation-wikipedia-normalize-word (word &optional method)
   ;; $B%9%Z!<%9$O(B %20 $B$G$O$J$/!"%"%s%@!<%9%3%"$KJQ49$9$k(B
-  (replace-regexp-in-string " "
-			    "_"
-			    (cond
-			     ((memq method '(ja.wiktionary en.wiktionary))
-			      (if (and (> (length word) 1)
-				       (skk-ascii-char-p (aref word 0))
-				       (skk-lower-case-p (aref word 1)))
-				  ;; $BFsJ8;z$a$,(B lower case $B$J$i(B downcase
-				  (downcase word)
-				;; $B0lJ8;z$@$C$?$i85$N(B case
-				;; $BFsJ8;z$a$,(B upper case $B$J$i85$N(B case
-				;; $B1Q8l0J30$OL$BP1~(B
-				word))
-			      ((eq method 'upcase-initials)
-			       (upcase-initials word))
-			      (t
-			       (concat (vector (upcase (aref word 0)))
-				       (substring word 1))))))
+  (replace-regexp-in-string
+   " " "_"
+   (cond
+    ((memq method '(ja.wiktionary en.wiktionary))
+     (if (and (> (length word) 1)
+	      (skk-ascii-char-p (aref word 0))
+	      (skk-lower-case-p (aref word 1)))
+	 ;; $BFsJ8;z$a$,(B lower case $B$J$i(B downcase
+	 (downcase word)
+       ;; $B0lJ8;z$@$C$?$i85$N(B case
+       ;; $BFsJ8;z$a$,(B upper case $B$J$i85$N(B case
+       ;; $B1Q8l0J30$OL$BP1~(B
+       word))
+    ((eq method 'upcase-initials)
+     (upcase-initials word))
+    (t
+     (if (> (length word) 1)
+	 (concat (vector (upcase (aref word 0)))
+		 (substring word 1))
+       word)))))
 
 (defun skk-annotation-url-package-available-p ()
   (when (eq skk-annotation-url-package-available-p 'untested)
-    ;; Emacs 22 $B0J9_0J30$G(B URL $B%Q%C%1!<%8$r%F%9%H$9$k(B
+    ;; GNU Emacs 22 $B0J9_0J30$G(B URL $B%Q%C%1!<%8$r%F%9%H$9$k(B
     (cond
-     ((or (and (eq skk-emacs-type 'mule4)
-	       (string-lessp mule-version "4.1"))
-	  (and (not (and (featurep 'xemacs)
-			 (string< "21.5" emacs-version)))
-	       (not (ignore-errors
-		      (require 'un-define)))))
-      ;; Emacs 20.7 (MULE 4.0) $B$G$O%5%]!<%H$7$J$$(B
-      ;; Emacs 20.7 (MULE 4.1) $B$OGS=|$7$J$$$G$*$/(B
+     ((and (featurep 'xemacs)
+	   (= emacs-major-version 21)
+	   (= emacs-minor-version 4)
+	   (not (featurep 'un-define)))
+      ;; XEmacs 21.4 $B$G(B Mule-UCS $B$b$J$$>l9g(B
       (setq skk-annotation-url-package-available-p nil))
      (t
       ;; Emacs 21 $B$H(B XEmacs
