@@ -6,9 +6,9 @@
 
 ;; Author: Masahiko Sato <masahiko@kuis.kyoto-u.ac.jp>
 ;; Maintainer: SKK Development Team <skk@ring.gr.jp>
-;; Version: $Id: skk-comp.el,v 1.81 2010/08/24 11:37:41 skk-cvs Exp $
+;; Version: $Id: skk-comp.el,v 1.84 2010/11/11 15:04:11 skk-cvs Exp $
 ;; Keywords: japanese, mule, input method
-;; Last Modified: $Date: 2010/08/24 11:37:41 $
+;; Last Modified: $Date: 2010/11/11 15:04:11 $
 
 ;; This file is part of Daredevil SKK.
 
@@ -64,8 +64,10 @@
   (skk-comp-do first silent))
 
 ;;;###autoload
-(defun skk-comp-do (first &optional silent)
+(defun skk-comp-do (first &optional silent set-this-command)
   ;; main completion engine.
+  (when set-this-command
+    (setq this-command 'skk-comp-do))
   (let ((inhibit-quit t)
 	;; skk-num $B$,(B require $B$5$l$F$J$$$H(B
 	;; buffer-local $BCM$r2u$962$l$"$j!#(B
@@ -358,9 +360,11 @@
       c-word)))
 
 ;;;###autoload
-(defun skk-comp-previous ()
+(defun skk-comp-previous (&optional set-this-command)
   ;; skk-abbrev-comma, skk-insert-comma $B$N%5%V%k!<%A%s!#(B
   ;; $BD>A0$KJd40$r9T$C$?8+=P$7$rA^F~$9$k!#(B
+  (when set-this-command
+    (setq this-command 'skk-comp-do))
   (let ((inhibit-quit t)
 	(stack-length (length skk-comp-stack))
 	c-word)
@@ -388,11 +392,38 @@
 
 ;;;###autoload
 (defun skk-comp-previous/next (ch)
-  (setq this-command 'skk-comp-do)
-  (cond ((eq ch skk-next-completion-char)
-	 (skk-comp-do nil))
-	((eq ch skk-previous-completion-char)
-	 (skk-previous-completion))))
+  (cond
+    ((eq ch skk-next-completion-char)
+     (skk-comp-do nil nil t))
+    ((eq ch skk-previous-completion-char)
+     (skk-comp-previous t))))
+
+;;;###autoload
+(defun skk-try-completion (arg)
+  "$B"&%b!<%I$G8+=P$78l$NJd40$r9T$&!#(B
+$B$=$l0J30$N%b!<%I$G$O!"%*%j%8%J%k$N%-!<3d$jIU$1$N%3%^%s%I$r%(%_%e%l!<%H$9$k!#(B"
+  (interactive "P")
+  (skk-with-point-move
+   (if (eq skk-henkan-mode 'on)
+       (skk-comp (or arg
+		     (not (eq last-command 'skk-comp-do))))
+     (skk-emulate-original-map arg))))
+
+;;;###autoload
+(defun skk-comp-wrapper (&optional arg)
+  "Character $B$G$J$$%-!<$KJd40$r3d$jEv$F$k$?$a$N%3%^%s%I!#(B"
+  (interactive "p")
+  (skk-bind-last-command-char skk-try-completion-char
+    (call-interactively #'skk-insert)))
+
+;;;###autoload
+(defun skk-previous-comp-maybe (&optional arg)
+  "Character $B$G$J$$%-!<$KJd40A08uJd$r3d$jEv$F$k$?$a$N%3%^%s%I!#(B
+$B"&%b!<%I$G$OJd40A08uJd!"$5$b$J$1$l$P%*%j%8%J%k$N%-!<Dj5A$r<B9T$9$k!#(B"
+  (interactive "P")
+  (if (eq skk-henkan-mode 'on) ;$B"&%b!<%I(B
+      (skk-comp-previous t)
+    (skk-emulate-original-map arg)))
 
 ;;;###autoload
 (defun skk-comp-by-history ()
@@ -562,7 +593,6 @@ WITHOUT-MIDASI $B$r;XDj$9$k$H8+=P$7$O>J$+$l$k!#(B"
 	    (setq words (nconc words tmp))))
 	words))))
 
-(defalias 'skk-previous-completion 'skk-comp-previous)
 (defalias 'skk-start-henkan-with-completion 'skk-comp-start-henkan)
 
 (run-hooks 'skk-comp-load-hook)
