@@ -1,13 +1,12 @@
 ;;; skk-macs.el --- macros and inline functions commonly used in SKK -*- coding: iso-2022-jp -*-
 
-;; Copyright (C) 1999-2010  SKK Development Team <skk@ring.gr.jp>
-;; Copyright (C) 1993, 2000 Free Software Foundation, Inc.
+;; Copyright (C) 1999-2010 SKK Development Team <skk@ring.gr.jp>
+;; Copyright (C) 1993-2000 Free Software Foundation, Inc.
 
-;; Author: SKK Development Team <skk@ring.gr.jp>
 ;; Maintainer: SKK Development Team <skk@ring.gr.jp>
-;; Version: $Id: skk-macs.el,v 1.145 2010/09/03 20:40:45 skk-cvs Exp $
+;; Version: $Id: skk-macs.el,v 1.158 2010/11/13 11:47:14 skk-cvs Exp $
 ;; Keywords: japanese, mule, input method
-;; Last Modified: $Date: 2010/09/03 20:40:45 $
+;; Last Modified: $Date: 2010/11/13 11:47:14 $
 
 ;; This file is part of Daredevil SKK.
 
@@ -32,65 +31,25 @@
 
 (eval-when-compile
   (defvar mule-version)
-  (defvar skk-e21-modeline-property))
+  (defvar skk-emacs-modeline-property))
 
 (eval-when-compile
-  (require 'advice)
-  (require 'static))
+  (require 'advice))
 
 (eval-and-compile
   (require 'skk-vars))
 
 ;;;; macros
 
-(put 'ignore-errors 'defmacro-maybe (not (featurep 'xemacs)))
-(defmacro-maybe ignore-errors (&rest body)
-  "Execute FORMS; if an error occurs, return nil.
+(when (eval-when-compile (and skk-running-gnu-emacs
+			      (<= emacs-major-version 22)))
+  (defmacro ignore-errors (&rest body)
+    "Execute FORMS; if an error occurs, return nil.
 Otherwise, return result of last FORM."
-  `(condition-case nil
-       (progn
-	 ,@body)
-     (error nil)))
-
-;;;###autoload
-(put 'dolist 'lisp-indent-function 1)
-(put 'dolist 'defmacro-maybe (eq skk-emacs-type 'mule4))
-
-;; From GNU Emacs 21.
-(defmacro-maybe dolist (spec &rest body)
-  "(dolist (VAR LIST [RESULT]) BODY...): loop over a list.
-Evaluate BODY with VAR bound to each car from LIST, in turn.
-Then evaluate RESULT to get return value, default nil."
-  (let ((temp (make-symbol "--dolist-temp--")))
-    (list 'let (list (list temp (nth 1 spec)) (car spec))
-	  (list 'while temp
-		(list 'setq (car spec) (list 'car temp))
-		(cons 'progn
-		      (append body
-			      (list (list 'setq temp (list 'cdr temp))))))
-	  (if (cdr (cdr spec))
-	      (cons 'progn
-		    (cons (list 'setq (car spec) nil) (cdr (cdr spec))))))))
-
-;;;###autoload
-(put 'dotimes 'lisp-indent-function 1)
-(put 'dotimes 'defmacro-maybe (eq skk-emacs-type 'mule4))
-
-;; From GNU Emacs 21.
-(defmacro-maybe dotimes (spec &rest body)
-  "(dotimes (VAR COUNT [RESULT]) BODY...): loop a certain number of times.
-Evaluate BODY with VAR bound to successive integers running from 0,
-inclusive, to COUNT, exclusive.  Then evaluate RESULT to get
-the return value (nil if RESULT is omitted)."
-  (let ((temp (make-symbol "--dotimes-temp--")))
-    (list 'let (list (list temp (nth 1 spec)) (list (car spec) 0))
-	   (list 'while (list '< (car spec) temp)
-		 (cons 'progn
-		       (append body (list (list 'setq (car spec)
-						(list '1+ (car spec)))))))
-	   (if (cdr (cdr spec))
-	       (car (cdr (cdr spec)))
-	     nil))))
+    `(condition-case nil
+	 (progn
+	   ,@body)
+       (error nil))))
 
 (defmacro skk-defadvice (function &rest everything-else)
   "Defines a piece of advice for FUNCTION (a symbol).
@@ -221,8 +180,8 @@ MARKER $B$,(B nil $B$@$C$?$i!"?75,%^!<%+!<$r:n$C$FBeF~$9$k!#(B"
 (def-edebug-spec skk-with-point-move t)
 
 (defmacro skk-face-on (object start end face &optional priority)
-  (static-cond
-   ((featurep 'xemacs)
+  (cond
+   ((eval-when-compile (featurep 'xemacs))
     `(let ((inhibit-quit t))
        (if (not (extentp ,object))
 	   (progn
@@ -265,35 +224,77 @@ MARKER $B$,(B nil $B$@$C$?$i!"?75,%^!<%+!<$r:n$C$FBeF~$9$k!#(B"
 
 ;;; functions.
 ;; version dependent
+;; Many functions are derived from emu (APEL).
 
-;; From XEmacs 21.5.
-(defun-maybe propertize (string &rest properties)
-  "Return a copy of STRING with text properties added.
-First argument is the string to copy.
-Remaining arguments form a sequence of PROPERTY VALUE pairs for text
-properties to add to the result."
-  (let ((str (copy-sequence string)))
-    (add-text-properties 0 (length str)
-			 properties
-			 str)
-    str))
+(when (eval-when-compile (and skk-running-gnu-emacs
+			      (<= emacs-major-version 22)))
+  (defalias 'characterp 'char-valid-p))
 
-(defun-maybe substring-no-properties (string &optional from to)
-  "Return a substring of string, without text properties.
+(when (eval-when-compile skk-running-gnu-emacs)
+  (defalias 'int-char 'identity))
+
+(when (eval-when-compile skk-running-gnu-emacs)
+  (defun string-to-char-list (string)
+    "Return a list of which elements are characters in the STRING."
+    (mapcar #'identity string)))
+
+(when (eval-when-compile skk-running-gnu-emacs)
+  (defalias 'string-to-int-list 'string-to-char-list))
+
+(when (eval-when-compile (featurep 'xemacs))
+  (defun string-to-int-list (str)
+    (mapcar #'char-int str)))
+
+(when (eval-when-compile skk-running-gnu-emacs)
+  (defun character-to-event (ch)
+    "Convert keystroke CH into an event structure, replete with bucky bits.
+Note that CH (the keystroke specifier) can be an integer, a character
+or a symbol such as 'clear."
+    ch))
+
+(when (eval-when-compile skk-running-gnu-emacs)
+  (defun event-to-character (event)
+    "Return the character approximation to the given event object.
+If the event isn't a keypress, this returns nil."
+    (cond
+     ((symbolp event)
+      ;; mask is (BASE-TYPE MODIFIER-BITS) or nil.
+      (let ((mask (get event 'event-symbol-element-mask)))
+	(if mask
+	    (let ((base (get (car mask) 'ascii-character)))
+	      (if base
+		  (logior base (car (cdr mask))))))))
+     ((integerp event)
+      event))))
+
+(when (eval-when-compile skk-running-gnu-emacs)
+  (defun cancel-undo-boundary ()
+    "Cancel undo boundary."
+    (if (and (consp buffer-undo-list)
+	     (null (car buffer-undo-list)))
+	(setq buffer-undo-list (cdr buffer-undo-list)))))
+
+(unless (eval-when-compile (and skk-running-gnu-emacs
+				(>= emacs-major-version 22)))
+  (defun substring-no-properties (string &optional from to)
+    "Return a substring of string, without text properties.
 It starts at index from and ending before to.
 to may be nil or omitted; then the substring runs to the end of string.
 If from is nil or omitted, the substring starts at the beginning of string.
 If from or to is negative, it counts from the end.
 
 With one argument, just copy string without its properties."
-  (let ((substr (copy-sequence (substring string (or from 0) to))))
-    (set-text-properties 0 (length substr) nil substr)
-    substr))
+    (let ((substr (copy-sequence (substring string (or from 0) to))))
+      (set-text-properties 0 (length substr) nil substr)
+      substr)))
 
 ;; From GNU Emacs 22.1.
-(defun-maybe replace-regexp-in-string (regexp rep string &optional
-					      fixedcase literal subexp start)
-  "Replace all matches for REGEXP with REP in STRING.
+(when (eval-when-compile (and (featurep 'xemacs)
+			      (= emacs-major-version 21)
+			      (= emacs-minor-version 4)))
+  (defun replace-regexp-in-string (regexp rep string &optional
+						fixedcase literal subexp start)
+    "Replace all matches for REGEXP with REP in STRING.
 
 Return a new string containing the replacements.
 
@@ -313,133 +314,138 @@ and replace a sub-expression, e.g.
     => \" bar foo\"
 "
 
-  ;; To avoid excessive consing from multiple matches in long strings,
-  ;; don't just call `replace-match' continually.  Walk down the
-  ;; string looking for matches of REGEXP and building up a (reversed)
-  ;; list MATCHES.  This comprises segments of STRING which weren't
-  ;; matched interspersed with replacements for segments that were.
-  ;; [For a `large' number of replacements it's more efficient to
-  ;; operate in a temporary buffer; we can't tell from the function's
-  ;; args whether to choose the buffer-based implementation, though it
-  ;; might be reasonable to do so for long enough STRING.]
-  (let ((l (length string))
-	(start (or start 0))
-	matches str mb me)
-    (save-match-data
-      (while (and (< start l) (string-match regexp string start))
-	(setq mb (match-beginning 0)
-	      me (match-end 0))
-	;; If we matched the empty string, make sure we advance by one char
-	(when (= me mb) (setq me (min l (1+ mb))))
-	;; Generate a replacement for the matched substring.
-	;; Operate only on the substring to minimize string consing.
-	;; Set up match data for the substring for replacement;
-	;; presumably this is likely to be faster than munging the
-	;; match data directly in Lisp.
-	(string-match regexp (setq str (substring string mb me)))
-	(setq matches
-	      (cons (replace-match (if (stringp rep)
-				       rep
-				     (funcall rep (match-string 0 str)))
-				   fixedcase literal str subexp)
-		    (cons (substring string start mb)       ; unmatched prefix
-			  matches)))
-	(setq start me))
-      ;; Reconstruct a string from the pieces.
-      (setq matches (cons (substring string start l) matches)) ; leftover
-      (apply #'concat (nreverse matches)))))
-
-(static-unless (featurep 'xemacs)
-  (fmakunbound 'next-command-event))
+    ;; To avoid excessive consing from multiple matches in long strings,
+    ;; don't just call `replace-match' continually.  Walk down the
+    ;; string looking for matches of REGEXP and building up a (reversed)
+    ;; list MATCHES.  This comprises segments of STRING which weren't
+    ;; matched interspersed with replacements for segments that were.
+    ;; [For a `large' number of replacements it's more efficient to
+    ;; operate in a temporary buffer; we can't tell from the function's
+    ;; args whether to choose the buffer-based implementation, though it
+    ;; might be reasonable to do so for long enough STRING.]
+    (let ((l (length string))
+	  (start (or start 0))
+	  matches str mb me)
+      (save-match-data
+	(while (and (< start l) (string-match regexp string start))
+	  (setq mb (match-beginning 0)
+		me (match-end 0))
+	  ;; If we matched the empty string, make sure we advance by one char
+	  (when (= me mb) (setq me (min l (1+ mb))))
+	  ;; Generate a replacement for the matched substring.
+	  ;; Operate only on the substring to minimize string consing.
+	  ;; Set up match data for the substring for replacement;
+	  ;; presumably this is likely to be faster than munging the
+	  ;; match data directly in Lisp.
+	  (string-match regexp (setq str (substring string mb me)))
+	  (setq matches
+		(cons (replace-match (if (stringp rep)
+					 rep
+				       (funcall rep (match-string 0 str)))
+				     fixedcase literal str subexp)
+		      (cons (substring string start mb)      ; unmatched prefix
+			    matches)))
+	  (setq start me))
+	;; Reconstruct a string from the pieces.
+	(setq matches (cons (substring string start l) matches)) ; leftover
+	(apply #'concat (nreverse matches))))))
 
 ;; For GNU Emacs.
-(defun-maybe next-command-event (&optional event prompt)
-  "Read an event object from the input stream.
+(when (eval-when-compile skk-running-gnu-emacs)
+  (defun next-command-event (&optional event prompt)
+    "Read an event object from the input stream.
 If EVENT is non-nil, it should be an event object and will be filled
 in and returned; otherwise a new event object will be created and
 returned.
 If PROMPT is non-nil, it should be a string and will be displayed in
 the echo area while this function is waiting for an event."
-  (read-event prompt))
+    (read-event prompt)))
 
-;; For GNU Emacs 20.7.
-(defalias-maybe 'plist-member 'widget-plist-member)
+(when (eval-when-compile (featurep 'xemacs))
+  (defun set-buffer-multibyte (flag)
+    "Set the multibyte flag of the current buffer to FLAG.
+If FLAG is t, this makes the buffer a multibyte buffer.
+If FLAG is nil, this makes the buffer a single-byte buffer.
+The buffer contents remain unchanged as a sequence of bytes
+but the contents viewed as characters do change.
+\[Emacs 20.3 emulating function]"
+  flag))
 
-(defsubst skk-sit-for (seconds &optional nodisplay)
+(defun skk-sit-for (seconds &optional nodisplay)
   "`sit-for' $B$N(B Emacsen $B$K$h$k0c$$$r5[<}$9$k!#(B"
-  (static-cond
-   ((featurep 'xemacs)
+  (cond
+   ((eval-when-compile (featurep 'xemacs))
     (sit-for seconds nodisplay))
-   ((< emacs-major-version 22)
+   ((eval-when-compile (= emacs-major-version 21))
+    ;; GNU Emacs 21
     (sit-for seconds nil nodisplay))
    (t
+    ;; GNU Emacs 22.1 or later
     (sit-for seconds nodisplay))))
 
-(defsubst skk-ding (&optional arg sound device)
+(defun skk-ding (&optional arg sound device)
   "`ding' $B$N(B Emacsen $B$K$h$k0c$$$r5[<}$9$k!#(B"
-  (static-cond
-   ((featurep 'xemacs)
-     (ding arg sound device))
+  (cond
+   ((eval-when-compile (featurep 'xemacs))
+    (ding arg sound device))
    (t
     (ding arg))))
 
-(defsubst skk-color-cursor-display-p ()
-  (static-cond
-   ((featurep 'xemacs)
+(defun skk-color-cursor-display-p ()
+  (cond
+   ((eval-when-compile (featurep 'xemacs))
     (eq (device-class (selected-device)) 'color))
-   ((fboundp 'x-display-color-p)
-    ;; FSF Emacs on X Window System.
-    (and window-system (x-display-color-p)))))
+   ((eval-when-compile skk-running-gnu-emacs)
+    (and (skk-find-window-system)
+	 (fboundp 'x-display-color-p)
+	 (x-display-color-p)))))
 
-(defsubst skk-char-to-unibyte-string (char)
+(defun skk-char-to-unibyte-string (char)
   (ignore-errors
-    (static-cond
-     ((and (string-match "^GNU" (emacs-version))
-	   (>= emacs-major-version 23))
+    (cond
+     ((eval-when-compile (and skk-running-gnu-emacs
+			      (>= emacs-major-version 23)))
+      ;; GNU Emacs 23.1 or later
       (string-make-unibyte (char-to-string char)))
      (t
       (char-to-string char)))))
 
-(defsubst skk-ascii-char-p (char)
-  (static-cond
-   ((and (string-match "^GNU" (emacs-version))
-	 (>= emacs-major-version 23))
+(defun skk-ascii-char-p (char)
+  (cond
+   ((eval-when-compile (and skk-running-gnu-emacs
+			    (>= emacs-major-version 23)))
+    ;; GNU Emacs 23.1 or later
     (eq (char-charset char skk-charset-list) 'ascii))
    (t
     (eq (char-charset char) 'ascii))))
 
-(defsubst skk-jisx0208-p (char)
-  (static-cond
-   ((and (string-match "^GNU" (emacs-version))
-	 (>= emacs-major-version 23))
+(defun skk-jisx0208-p (char)
+  (cond
+   ((eval-when-compile (and skk-running-gnu-emacs
+			    (>= emacs-major-version 23)))
+    ;; GNU Emacs 23.1 or later
     (eq (char-charset char skk-charset-list) 'japanese-jisx0208))
    (t
     (eq (char-charset char) 'japanese-jisx0208))))
 
-(defsubst skk-jisx0213-p (char)
-  (static-cond
-   ((and (string-match "^GNU" (emacs-version))
-	   (>= emacs-major-version 23))
-       (memq (char-charset char skk-charset-list)
-	     '(japanese-jisx0213.2004-1
-	       japanese-jisx0213-1
-	       japanese-jisx0213-2)))
+(defun skk-jisx0213-p (char)
+  (cond
+   ((eval-when-compile (and skk-running-gnu-emacs
+			    (>= emacs-major-version 23)))
+    ;; GNU Emacs 23.1 or later
+    (memq (char-charset char skk-charset-list)
+	  '(japanese-jisx0213-1
+	    japanese-jisx0213.2004-1
+	    japanese-jisx0213-2)))
    (t
     (and (featurep 'jisx0213)
 	 (memq (char-charset char)
 	       '(japanese-jisx0213-1 japanese-jisx0213-2))))))
 
-(defsubst skk-char-octet (ch &optional n)
-  (or (nth (if n
-	       (1+ n)
-	     1)
-	   (skk-split-char ch))
-      0))
-
 (defun skk-split-char (ch)
-  (static-cond
-   ((and (string-match "^GNU" (emacs-version))
-	 (>= emacs-major-version 23))
+  (cond
+   ((eval-when-compile (and skk-running-gnu-emacs
+			    (>= emacs-major-version 23)))
     ;; C $B$N(B split-char $B$HF1MM$N5!G=$@$,!"(Bchar-charset $B$N8F=P$7$K$*$$$F(B
     ;; $BJ8;z=89g$NA*Br;h$r(B skk-charset-list $B$K4^$^$l$k$b$N$K@)8B$9$k!#(B
     ;; $B$3$l$ONc$($P!"(Bjapanese-jisx0208 $B$NJ8;z$,(B unicode-bmp $B$KB0$9$k!"(B
@@ -459,22 +465,21 @@ the echo area while this function is waiting for an event."
     (split-char ch))))
 
 ;; this one is called once in skk-kcode.el, too.
-(defsubst skk-charsetp (object)
-  (static-cond
-   ((featurep 'xemacs)
+(defun skk-charsetp (object)
+  (cond
+   ((eval-when-compile (featurep 'xemacs))
     (find-charset object))
    (t
-    ;; FSF Emacs 20 or later.
     (charsetp object))))
 
 (defun skk-indicator-to-string (indicator &optional no-properties)
   "SKK $B%$%s%8%1!<%?7?%*%V%8%'%/%H(B INDICATOR $B$rJ8;zNs$KJQ49$9$k!#(B"
-  (static-cond
-   ((featurep 'xemacs)
+  (cond
+   ((eval-when-compile (featurep 'xemacs))
     (if (stringp indicator)
 	indicator
       (cdr indicator)))
-   ((>= emacs-major-version 21)
+   ((eval-when-compile (>= emacs-major-version 21))
     (if no-properties
 	(with-temp-buffer
 	  (insert indicator)
@@ -485,44 +490,38 @@ the echo area while this function is waiting for an event."
 
 (defun skk-mode-string-to-indicator (mode string)
   "$BJ8;zNs(B STRING $B$r(B SKK $B%$%s%8%1!<%?7?%*%V%8%'%/%H$KJQ49$9$k!#(B"
-  (static-cond
-   ((featurep 'xemacs)
+  (cond
+   ((eval-when-compile (featurep 'xemacs))
     (cons (cdr (assq mode skk-xemacs-extent-alist))
 	  string))
-   ((>= emacs-major-version 21)
+   (t
     (if (and window-system
 	     (not (eq mode 'default)))
 	(apply 'propertize string
-	       (cdr (assq mode skk-e21-property-alist)))
-      string))
-   (t
-    string)))
+	       (cdr (assq mode skk-emacs-property-alist)))
+      string))))
 
-(defsubst skk-local-variable-p (variable &optional buffer afterset)
+(defun skk-local-variable-p (variable &optional buffer afterset)
   "Non-nil if VARIABLE has a local binding in buffer BUFFER.
 BUFFER defaults to the current buffer."
-  (static-cond
-   ((featurep 'xemacs)
+  (cond
+   ((eval-when-compile (featurep 'xemacs))
     (local-variable-p variable (or buffer (current-buffer)) afterset))
    (t
     (local-variable-p variable (or buffer (current-buffer))))))
 
-(defsubst skk-face-proportional-p (face)
-  (static-cond
-   ((featurep 'xemacs)
+(defun skk-face-proportional-p (face)
+  (cond
+   ((eval-when-compile (featurep 'xemacs))
     (face-proportional-p face))
-   ((>= emacs-major-version 21)
+   (t
     (or (face-equal face 'variable-pitch)
 	(eq (face-attribute face :inherit) 'variable-pitch)))))
 
 (defun skk-event-key (event)
   "$B%$%Y%s%H(B EVENT $B$rH/@8$7$?F~NO$N>pJs$r<hF@$9$k!#(B"
-  (static-cond
-   ((featurep 'xemacs)
-;    (let ((tmp (event-key event)))
-;      (if (symbolp tmp)
-;	  (vector tmp)
-;	event)))
+  (cond
+   ((eval-when-compile (featurep 'xemacs))
     (let ((char (event-to-character event t t t)))
       (if (characterp char)
 	  char
@@ -540,21 +539,30 @@ BUFFER defaults to the current buffer."
 ;; What is annoying is incompatibility between FSF Emacs and XEmacs.
 ;; In XEmacs, last-command-event is an event, not a character, whereas
 ;; last-command-char is a character equivalent to last-command-event.
-(defsubst skk-last-command-char ()
-  (static-cond
-   ((featurep 'xemacs)
+(defun skk-last-command-char ()
+  (cond
+   ((eval-when-compile (featurep 'xemacs))
     last-command-char)
    (t
     last-command-event)))
 
-(defsubst skk-set-last-command-char (char)
-  (let ((variable (static-cond
-		   ((featurep 'xemacs)
-		    'last-command-char)
-		   (t
-		    'last-command-event))))
+(defun skk-set-last-command-char (char)
+  (let ((variable (if (featurep 'xemacs)
+		      'last-command-char
+		    'last-command-event)))
     (set variable char)))
 
+(defun skk-region-active-p ()
+  (cond
+   ((eval-when-compile (and skk-running-gnu-emacs
+			    (>= emacs-major-version 23)))
+    (use-region-p))
+   ((eval-when-compile (featurep 'xemacs))
+    (region-active-p))
+   (t
+    ;; GNU Emacs 21 and 22.
+    (and transient-mark-mode mark-active))))
+
 (put 'skk-bind-last-command-char 'lisp-indent-function 1)
 (defmacro skk-bind-last-command-char (char &rest body)
   (let ((variable (cond ((featurep 'xemacs)
@@ -566,20 +574,28 @@ BUFFER defaults to the current buffer."
 	 ,@body))))
 
 ;;; version independent
-(defsubst skk-cursor-set (&optional color force)
+
+(defsubst skk-char-octet (ch &optional n)
+  (or (nth (if n
+	       (1+ n)
+	     1)
+	   (skk-split-char ch))
+      0))
+
+(defun skk-cursor-set (&optional color force)
   (unless (skk-color-cursor-display-p)
     (setq skk-use-color-cursor nil))
   (when (or skk-use-color-cursor
 	    force)
     (skk-cursor-set-1 color)))
 
-(defsubst skk-cursor-off ()
+(defun skk-cursor-off ()
   (unless (skk-color-cursor-display-p)
     (setq skk-use-color-cursor nil))
   (when skk-use-color-cursor
     (skk-cursor-off-1)))
 
-(defsubst skk-modify-indicator-alist (mode string)
+(defun skk-modify-indicator-alist (mode string)
   (setcdr (assq mode skk-indicator-alist)
 	  (cons string (skk-mode-string-to-indicator mode string))))
 
@@ -598,7 +614,7 @@ BUFFER defaults to the current buffer."
     (force-mode-line-update)))
 
 ;; $B%D%j!<$K%"%/%;%9$9$k$?$a$N%$%s%?!<%U%'!<%9(B
-(defsubst skk-make-rule-tree (char prefix nextstate kana branch-list)
+(defun skk-make-rule-tree (char prefix nextstate kana branch-list)
   (list char
 	prefix
 	(if (string= nextstate "")
@@ -665,9 +681,7 @@ BUFFER defaults to the current buffer."
 		 skk-current-rule-tree nil))))
   (when clean
     (setq skk-prefix ""
-	  skk-current-rule-tree nil)) ; fail safe
-  (static-when (eq skk-emacs-type 'mule4)
-    (redraw-frame (selected-frame))))
+	  skk-current-rule-tree nil))) ; fail safe
 
 (defun skk-kana-cleanup (&optional force)
   (let ((data (cond
@@ -789,20 +803,25 @@ BUFFER defaults to the current buffer."
   (skk-cursor-set))
 
 (defsubst skk-in-minibuffer-p ()
-  "$B%+%l%s%H%P%C%U%!$,%_%K%P%C%U%!$+$I$&$+$r%A%'%C%/$9$k!#(B"
+  "$B%+%l%s%H%P%C%U%!$,%_%K%P%C%U%!$G$"$l$P(B t $B$rJV$9!#(B"
   (eq (current-buffer) (window-buffer (minibuffer-window))))
 
 (defun skk-window-body-height ()
-  (static-if (fboundp 'window-body-height) ; emacs21 $B$K$O$J$$(B
-      (window-body-height)
+  (cond
+   ((eval-when-compile (featurep 'xemacs))
+    nil) ; XEmacs $B$G%5%]!<%H$5$l$J$$5!G=(B
+   ((eval-when-compile (and skk-running-gnu-emacs
+			      (>= emacs-major-version 22)))
+    (window-body-height)) ; emacs21 $B$K$O$J$$(B
+   (t
     (- (window-height)
        (if mode-line-format 1 0)
-       (static-cond ((not (boundp 'header-line-format)) 0) ; emacs20 $B$K$O$J$$(B
-		    (t (if header-line-format 1 0))))))
+       (if header-line-format 1 0)))))
 
 (defun skk-screen-column ()
   "$B%9%/%j!<%s9T$+$iF@$?%+!<%=%k0LCV$N7e?t$rJV$9!#(B
-$B%F%-%9%H9T!J2~9TJ8;z$G6h@Z$i$l$?%F%-%9%H!K$,%&%#%s%I%&I}$r1[$($F@^$jJV$7$FI=<($5$l$F$$$k>l9g$K$bBP1~$9$k!#(B"
+$B%F%-%9%H9T!J2~9TJ8;z$G6h@Z$i$l$?%F%-%9%H!K$,%&%#%s%I%&I}$r1[$($F@^$jJV$7$FI=<((B
+$B$5$l$F$$$k>l9g$K$bBP1~$9$k!#(B"
   (- (current-column)
      (save-excursion
        (vertical-motion 0)		;$B%9%/%j!<%s9T$N9TF,$K0\F0$9$k(B
@@ -811,7 +830,8 @@ BUFFER defaults to the current buffer."
 
 (defun skk-move-to-screen-column (col)
   "$B%9%/%j!<%s9T$+$i8+$?(B COL $B7e0LCV$K%]%$%s%H$r0\F0$9$k!#(B
-$B%F%-%9%H9T!J2~9TJ8;z$G6h@Z$i$l$?%F%-%9%H!K$,%&%#%s%I%&I}$r1[$($F@^$jJV$7$FI=<($5$l$F$$$k>l9g$K$bBP1~$9$k$,!"2~9TJ8;z$r1[$($k0\F0$O9T$o$J$$!#(B"
+$B%F%-%9%H9T!J2~9TJ8;z$G6h@Z$i$l$?%F%-%9%H!K$,%&%#%s%I%&I}$r1[$($F@^$jJV$7$FI=<((B
+$B$5$l$F$$$k>l9g$K$bBP1~$9$k$,!"2~9TJ8;z$r1[$($k0\F0$O9T$o$J$$!#(B"
   (move-to-column (+ (current-column)	;$B%F%-%9%H9T$+$i8+$?7e?t(B
 		     (- col (skk-screen-column))))
   (skk-screen-column))
@@ -832,22 +852,20 @@ BUFFER defaults to the current buffer."
 				 'skk-prefix-katakana-face)
 				(skk-jisx0201-mode
 				 'skk-prefix-jisx0201-face)))))
-      (overlay-put skk-prefix-overlay 'after-string prefix)
-      (static-when (eq skk-emacs-type 'mule4)
-	(redraw-frame (selected-frame))))))
+      (overlay-put skk-prefix-overlay 'after-string prefix))))
 
 (defsubst skk-string-lessp-in-coding-system (str1 str2 coding-system)
   (string< (encode-coding-string str1 coding-system)
 	   (encode-coding-string str2 coding-system)))
 
-(defsubst skk-string< (str1 str2)
+(defun skk-string< (str1 str2)
   "STR1 $B$H(B STR2 $B$H$rHf3S$9$k!#(B
 $BFbIt%3!<%I$,(B emacs-mule $B$G$J$$$J$I(B `stringp' $B$NJV$jCM$,0[$J$k(B Emacs $B$K(B
 $BBP$7$F(B emacs-mule $B$N(B encoded string $B$KJQ49$7$FHf3S$9$k!#(B
 $BHf3S$N7k2L(B str1 < str2 $B$J$i$P(B t $B$rJV$9!#(B"
-  (static-cond
-   ((and (string-match "^GNU" (emacs-version))
-	 (string< "6.0" mule-version))
+  (cond
+   ((eval-when-compile (and skk-running-gnu-emacs
+			    (>= emacs-major-version 23)))
     ;; Emacs with coding system utf-8-emacs
     (skk-string-lessp-in-coding-system str1 str2 'emacs-mule))
    (t
@@ -890,9 +908,10 @@ BUFFER defaults to the current buffer."
 ;; <nextstate>    ::= <$B1Q>.J8;zJ8;zNs(B> | nil
 
 (defsubst skk-make-raw-arg (arg)
-  (cond ((= arg 1) nil)
-	((= arg -1) '-)
-	((numberp arg) (list arg))))
+  (case arg
+    (1 nil)
+    (-1 '-)
+    (t (if (numberp arg) (list arg) nil))))
 
 (defsubst skk-unread-event (event)
   "Unread single EVENT."
@@ -992,6 +1011,37 @@ MAP $B$OF~NO$,=q$+$l$F$$$k%-!<%^%C%W$r;XDj$9$k$,!";XDj$5$l$J$1$l$P(B
 			(key-description k))
 		    keys))))
 
+(defun skk-update-minor-mode-map-alist (mode map)
+  (let ((element (assq mode minor-mode-map-alist)))
+    (if element
+	(setcdr element map)
+      (add-to-list 'minor-mode-map-alist (cons mode map)))))
+
+;; Functions from alist.el (APEL)
+(defun skk-put-alist (key value alist)
+  "Set cdr of an element (KEY . ...) in ALIST to VALUE and return ALIST.
+If there is no such element, create a new pair (KEY . VALUE) and
+return a new alist whose car is the new pair and cdr is ALIST."
+  (let ((elm (assoc key alist)))
+    (if elm
+	(progn
+	  (setcdr elm value)
+	  alist)
+      (cons (cons key value) alist))))
+
+(defun skk-del-alist (key alist)
+  "Delete an element whose car equals KEY from ALIST.
+Return the modified ALIST."
+  (let ((pair (assoc key alist)))
+    (if pair
+	(delq pair alist)
+      alist)))
+
+(defun skk-remove-alist (symbol key)
+  "Delete an element whose car equals KEY from the alist bound to SYMBOL."
+  (and (boundp symbol)
+       (set symbol (skk-del-alist key (symbol-value symbol)))))
+
 (provide 'skk-macs)
 
 ;;; skk-macs.el ends here
